/**
 *  Copyright notice
 *  
 *  This file is part of the Processing sketch `Michelson Interferometer' 
 *  http://www.gwoptics.org/processing/michelson01/
 *
 *  Copyright (C) 2009 onwards Charlotte Bond and Andreas Freise
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 *  MA  02110-1301, USA.
 */

class BeamSplitter extends PApplet implements PConstants {

  // Primary variables
  PApplet parent;
  int sketch_width, sketch_height;

  // Secondary variables
  PFont font;

  // Optics variables
  int beam_split_x;
  int beam_split_y, beam_split_length, beam_split_diag;
  int mirror2_x, mirror1_y, mirror_length;
  int mirror_tmp_pos1, mirror_tmp_pos2;
  float wavelength, k, n1, n_bs, amplitude;
  float r_bs, t_bs, r_m1, r_m2, wt;

  float speed, temp_speed;

  boolean paused, variable_changed;

  float pi=(float)Math.PI;

  // lookup for sinus values
  int nlookup=10000;
  float [] sinlookup = new float[nlookup];


  // --------------------------------------------------------- Class definition --------------------------------


  BeamSplitter(PApplet t_parent, int t_sketch_width, int t_sketch_height) {
    if (t_parent == null)
      throw new RuntimeException("Parent object cannot be null");

    parent = t_parent;
    sketch_width = t_sketch_width;
    sketch_height = t_sketch_height;


    beam_split_x = round(sketch_width/2);
    beam_split_y = round(sketch_height/2);

    beam_split_length = round(sketch_height/5);
    beam_split_diag=round(beam_split_length*0.5*(float)Math.sqrt(2));

    mirror1_y = 20;
    mirror2_x = beam_split_x+beam_split_y-mirror1_y; // arm length difference = 0
    mirror_length = round(sketch_width/7);
    mirror_tmp_pos1=round(sketch_width/4);
    mirror_tmp_pos2=ceil(mirror_length/2);

    wavelength = 150;
    k = (float)Math.PI*2/wavelength;
    n1 = 1.0;
    n_bs = 3.5;
    amplitude = 25;
    r_bs = 1/pow(2,0.5);
    t_bs = pow(1-pow(r_bs,2),0.5);
    r_m1 = 1.0;
    r_m2 = 1.0;

    wt = 0.0;
    speed = 0.2;
    temp_speed = speed;

    paused = false;
    variable_changed = false;
  }    

  // ---------------------------------------------------------------- Setup --------------------------------

  void setup() {
    System.out.println(sketch_width + sketch_height);
    size(sketch_width,sketch_height,JAVA2D);
    noLoop();
    frameRate(30);

    font = createFont("Verdana",14,true);
    for (int i = 0; i < nlookup; i++) {
      sinlookup[i] = cos(k*i)*amplitude;
    }
  }


  // ---------------------------------------------------------------- Draw --------------------------------


  void draw() {
    if (paused == false)
    {
      calcAndDraw();
    }
    else if (variable_changed == true && paused == true) {
      calcAndDraw();
      variable_changed = false;
    }
  }


  // ------------------------------------------------------------- Calc and Draw --------------------------------

  void calcAndDraw() {
    background(240,240,245);
    // increase time and compute omega*t modulo 2PI
    wt += speed;
    //wt=0;
    wt = wt%(2*(float)Math.PI);

    strokeWeight(1);

    //void drawWaveHor(float amp, int phase, color col, int y, int startx)
    stroke(0);
    smooth();
    // incoming wave
    drawWaveHorBS(1,wt/k,beam_split_y,0);

    // not undertsood offset to create continous waves at mirror surfaces
    // needs to be adjusted when wavelength is changed
    float myoffset=100;


    // transmitted wave
    drawWaveHorBS(t_bs,wt/k-mirror2_x,beam_split_y,mirror2_x);
    stroke(0,0,200);
    drawWaveHorBS(t_bs*r_m2,-wt/k+mirror2_x+myoffset,beam_split_y,mirror2_x);
    stroke(0,0,200,50);
    float phs_bs2=2*mirror2_x-beam_split_x+sketch_height-beam_split_y-myoffset;
    drawWaveVerBS(t_bs*r_m2*r_bs,wt/k-phs_bs2,beam_split_x,sketch_height);
    stroke(0);
    float phs_m1=beam_split_x+(beam_split_y-mirror1_y);
    drawWaveVerBS(r_bs,-wt/k+phs_m1,beam_split_x,mirror1_y);
    stroke(0,200,0);
    drawWaveVerBS(r_bs*r_m1,wt/k-phs_m1+myoffset,beam_split_x,mirror1_y);
    float phs_bs1=beam_split_x+2*(beam_split_y-mirror1_y)+(sketch_height-beam_split_y)-myoffset;
    stroke(0,200,0,50);
    drawWaveVerBS(t_bs*r_bs*r_m1,wt/k-phs_bs1,beam_split_x,sketch_height);
    stroke(200,0,0);
    //drawDoubleWaveVerBS(r_bs*t_bs*r_m2,-wt/k-beam_split_x+2*mirror2_x+sketch_height,r_bs*t_bs*r_m1,-wt/k-beam_split_x+sketch_height,beam_split_x,sketch_height);
    //drawDoubleWaveHorBS(t_bs*t_bs*r_m2,-wt/k-beam_split_x+2*mirror2_x,r_bs*r_bs*r_m1,-wt/k-beam_split_x+pi/k,beam_split_y,0);
    drawDoubleWaveVerBS(r_bs*t_bs*r_m2,wt/k-phs_bs2,r_bs*t_bs*r_m1,wt/k-phs_bs1,beam_split_x,sketch_height);
    float phs_bs3=2*mirror2_x-2*myoffset;
    float phs_bs4=beam_split_x+2*(beam_split_y-mirror1_y)+beam_split_x-2*myoffset+pi/k;
    drawDoubleWaveHorBS(t_bs*t_bs*r_m2,-wt/k+phs_bs3,r_bs*r_bs*r_m1,-wt/k+phs_bs4,beam_split_y,0);


    textAlign(LEFT);
    writeLabels("Beam-splitter",beam_split_x+beam_split_diag-30,beam_split_y-beam_split_diag-20);
    writeLabels("Second Mirror",mirror2_x-100+round(0.25*(sketch_width-mirror2_x)),beam_split_y+mirror_length/2+20);
    textAlign(RIGHT);
    writeLabels("First Mirror",beam_split_x-mirror_tmp_pos2-10,mirror1_y+2);
    stroke(255,0,0);
    strokeWeight(1.6);
    drawBeamSplitter();
    drawMirrors();
    drawLegend();
  }


  void drawWaveHorBS(float amp, float phase, int y, int startx)
  {
    int j=0;
    int i=startx;
    // println("phase "+phase);
    int ph=round(nlookup*.5-phase);
    if (beam_split_x>startx)
    {
      while(amp*sinlookup[j+ph]>i-beam_split_x) {
        line(i,y-amp*sinlookup[j+ph],i+1,y-amp*sinlookup[j+1+ph]);
        i++;
        j++;
      }
    }
    else
    {
      while(amp*sinlookup[j+ph]<i-beam_split_x) {
        line(i,y-amp*sinlookup[j+ph],i+1,y-amp*sinlookup[j+1+ph]);
        i--;
        j--;
      }
    }
  }


  void drawDoubleWaveHorBS(float amp1, float phase1, float amp2, float phase2, int y, int startx)
  {
    int j=0;
    int i=startx;
    int ph1=round(nlookup*.5-phase1);
    int ph2=round(nlookup*.5-phase2);
    if (beam_split_x>startx)
    {
      while(amp1*sinlookup[j+ph1]+amp2*sinlookup[j+ph2]>i-beam_split_x) {
        line(i,y-amp1*sinlookup[j+ph1]-amp2*sinlookup[j+ph2],i+1,y-amp1*sinlookup[j+1+ph1]-amp2*sinlookup[j+1+ph2]);
        i++;
        j++;
      }
    }
    else
    {
      while(amp1*sinlookup[j+ph1]+amp2*sinlookup[j+ph2]<i-beam_split_x) {
        line(i,y-amp1*sinlookup[j+ph1]-amp2*sinlookup[j+ph2],i+1,y-amp1*sinlookup[j+1+ph1]-amp2*sinlookup[j+1+ph2]);
        i--;
        j--;
      }
    }
  }

  void drawWaveVerBS(float amp, float phase, int x, int starty)
  {
    int j=0;
    int i=starty;
    int ph=round(nlookup*.5-phase);
    if (beam_split_y>starty)
    {
      while(amp*sinlookup[j+ph]>i-beam_split_y) {
        line(x-amp*sinlookup[j+ph],i,x-amp*sinlookup[j+1+ph],i+1);
        i++;
        j++;
      }
    }
    else
    {
      while(amp*sinlookup[j+ph]<i-beam_split_y) {
        line(x-amp*sinlookup[j+ph],i,x-amp*sinlookup[j+1+ph],i+1);
        i--;
        j--;
      }
    }
  }



  void drawDoubleWaveVerBS(float amp1, float phase1, float amp2, float phase2, int x, int starty)
  {
    int j=0;
    int i=starty;
    int ph1=round(nlookup*.5-phase1);
    int ph2=round(nlookup*.5-phase2);
    if (beam_split_y>starty)
    {
      while(amp1*sinlookup[j+ph1]+amp2*sinlookup[j+ph2]>i-beam_split_y) {
        line(x-amp1*sinlookup[j+ph1]-amp2*sinlookup[j+ph2],i,x-amp1*sinlookup[j+1+ph1]-amp2*sinlookup[j+1+ph2],i+1);
        i++;
        j++;
      }
    }
    else
    {
      while(amp1*sinlookup[j+ph1]+amp2*sinlookup[j+ph2]<i-beam_split_y) {
        line(x-amp1*sinlookup[j+ph1]-amp2*sinlookup[j+ph2],i,x-amp1*sinlookup[j+1+ph1]-amp2*sinlookup[j+1+ph2],i+1);
        i--;
        j--;
      }
    }
  }



  void drawBeamSplitter() {
    int bs_xstart = beam_split_x + beam_split_diag;
    int bs_ystart = beam_split_y - beam_split_diag;
    int bs_xend = beam_split_x - beam_split_diag;
    int bs_yend = beam_split_y + beam_split_diag;
    pushMatrix();
    //smooth();
    strokeWeight(6);
    stroke(0); 
    line(bs_xstart-1,bs_ystart,bs_xend-1,bs_yend);
    popMatrix();
  }

  void drawMirrors() {
    pushMatrix();
    //smooth();
    strokeWeight(6);
    stroke(0);
    line(mirror2_x,beam_split_y-mirror_length/2,mirror2_x,beam_split_y+mirror_length/2);
    line(beam_split_x-mirror_length/2,mirror1_y,beam_split_x+mirror_length/2,mirror1_y);
    popMatrix();
  } 


  void drawLegend() {
    pushMatrix();
    //smooth();
    strokeWeight(2);
    stroke(0);
    int arrx=100;
    int arry=beam_split_y-50;
    int tipx=150;
    line(arrx,arry,tipx,arry);
    line(tipx,arry,tipx-10,arry-4);
    line(tipx,arry,tipx-10,arry+4);
    writeLabels("input",arrx+40,arry-17);
    stroke(200,0,0);
    arry=beam_split_y+50;
    arrx=150;
    tipx=100;
    line(arrx,arry,tipx,arry);
    line(tipx,arry,tipx+10,arry-4);
    line(tipx,arry,tipx+10,arry+4);
    writeLabels("output",tipx+50,arry+27);

    arry=300;
    arrx=beam_split_x-55;
    int tipy=350;
    line(arrx,arry,arrx,tipy);
    line(arrx,tipy,arrx+4,tipy-10);
    line(arrx,tipy,arrx-4,tipy-10);
    writeLabels("output",arrx-20,arry+30);


    popMatrix();
  } 


  void writeLabels(String label, int x_position, int y_position) {
    fill(0);
    textFont(font,14);
    text(label,x_position,y_position);
  }



  // set functions

  void setCavity2Length(float cavity2_length) {
    int least = round(cavity2_length);
    mirror2_x = beam_split_x + least;
    variable_changed = true;
  }

  void setBSReflectivity(float t_r_bs) {
    r_bs = pow(t_r_bs,0.5);
    t_bs = pow(1-t_r_bs,0.5);
    variable_changed = true;
    //println("r "+r_bs+", t "+t_bs);
  }

  void setM1Reflectivity(float t_r_m1) {
    r_m1 = pow(t_r_m1,0.5);
    variable_changed = true;
  }

  void setM2Reflectivity(float t_r_m2) {
    r_m2 = pow(t_r_m2,0.5);
    variable_changed = true;
  }



  void em_pause() {
    speed = 0;
    paused = true;
  }

  void em_play() {
    speed = temp_speed;
    paused = false;
  }

  // get functions

  int getCavity2Length() {
    return mirror2_x - beam_split_x;
  }

  int getCavity1Length() {
    return beam_split_y-mirror1_y;
  }

  float getBSReflectivity() {
    return r_bs;
  }

  float getM1Reflectivity() {
    return r_m1;
  }

  float getM2Reflectivity() {
    return r_m2;
  }

  float getWavelength() {
    return wavelength;
  }



  public void keyPressed() {		
    if(key == ' ') {
      if (paused==true) {
        unpause();
      }   
      else {
        pause();
      }
    }
  }
}

