/**
 *  Copyright notice
 *  
 *  This file is part of the Processing sketch `Michelson Interferometer' 
 *  http://www.gwoptics.org/processing/michelson01/
 *
 *  Copyright (C) 2009 onwards Charlotte Bond and Andreas Freise
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 *  MA  02110-1301, USA.
 */


import guicomponents.*;
import org.gwoptics.ValueType;
import org.gwoptics.Logo;
import org.gwoptics.LogoSize;

//import fullscreen.*; 
//FullScreen fs;

PImage background_image;
PFont title_font;
float scale_h, scale_w;
BeamSplitter beamsplitter_sketch;
int bs_sketch_x=15;
int bs_sketch_y=5;
int bs_sketch_width=870;
int bs_sketch_height=400;

// Sliders to change optical paraameters
GWSlider cavity2_length_slider, reflectivity_bs_slider;
GWSlider reflectivity_m1_slider, reflectivity_m2_slider;

// Labels
GLabel cavity2_length_label, rbs_label, rm1_label, rm2_label, r_label;
GLabel cav_dif_label, cav_dif_unit_label;

GTextField dif_cav_dist_text;

boolean paused=false;

GCheckbox chkContinous;


Logo gw_logo;

void setup() {
  size(900,520,JAVA2D);
  smooth();
  frameRate(30);
  // enter fullscreen mode
  //  fs = new FullScreen(this);   
  //  fs.enter(); 
  //  fs.setResolution(1024,640);
  background_image = loadImage("layout2.png");

  // beamsplitter_sketch referencing this sketch.
  beamsplitter_sketch = new BeamSplitter(this,bs_sketch_width,bs_sketch_height);
  // Initialise
  beamsplitter_sketch.init();
  this.setLayout(null);

  beamsplitter_sketch.setBounds(bs_sketch_x,bs_sketch_y,beamsplitter_sketch.width,beamsplitter_sketch.height);

  this.add(beamsplitter_sketch);

  gw_logo = new Logo(this,width-560,height-40,true,LogoSize.Size35);

  int RSoffset1=width-450;  
  int RSoffset2=width-270;

  int labelyoffset=10;

  // ------------------------------------------ Continuous ---------------------------------------
  chkContinous = new GCheckbox(this,"pause",bs_sketch_x+260,bs_sketch_y+bs_sketch_height+labelyoffset,110);
  chkContinous.setSelected(false); 

  // ------------------------------------------ Arm length ---------------------------------------  
  cavity2_length_label = new GLabel(this,"Length of second arm",bs_sketch_x+10,bs_sketch_y+bs_sketch_height+labelyoffset,200);
  cavity2_length_slider = new GWSlider(this,"blue18px",bs_sketch_x+15,bs_sketch_y+bs_sketch_height+labelyoffset+25,200);
  cavity2_length_slider.setValueType(GWSlider.DECIMAL);
  cavity2_length_slider.setLimits(0.5f,80,bs_sketch_width/2-9);
  cavity2_length_slider.setRenderValueLabel(false);
  cavity2_length_slider.setValue(beamsplitter_sketch.getCavity2Length());

  dif_cav_dist_text = new GTextField(this,String.valueOf(0.0),bs_sketch_x+180,bs_sketch_y+bs_sketch_height+labelyoffset+70,50,12);

  cav_dif_label = new GLabel(this,"arm length difference [lambda]",bs_sketch_x+5,bs_sketch_y+bs_sketch_height+labelyoffset+70,200);

  // ------------------------------------------ Reflectivities ---------------------------------------

  r_label   = new GLabel(this,"Reflectivities", bs_sketch_x+width-350+25,bs_sketch_y+bs_sketch_height+labelyoffset,300);
  reflectivity_bs_slider = new GWSlider(this,"blue18px",bs_sketch_x+RSoffset1,bs_sketch_y+bs_sketch_height+labelyoffset+25,150);
  reflectivity_bs_slider.setValueType(GWSlider.DECIMAL);
  reflectivity_bs_slider.setLimits(0.8f,0f,1f);
  reflectivity_bs_slider.setRenderValueLabel(false);
  reflectivity_bs_slider.setValue(pow(beamsplitter_sketch.getBSReflectivity(),2));

  reflectivity_m1_slider = new GWSlider(this,"blue18px",bs_sketch_x+RSoffset2,bs_sketch_y+bs_sketch_height+labelyoffset+25,150);
  reflectivity_m1_slider.setValueType(GWSlider.DECIMAL);
  reflectivity_m1_slider.setLimits(0.5f,0f,1f);
  reflectivity_m1_slider.setRenderValueLabel(false);
  reflectivity_m1_slider.setValue(pow(beamsplitter_sketch.getM1Reflectivity(),2));

  reflectivity_m2_slider = new GWSlider(this,"blue18px",bs_sketch_x+RSoffset2,bs_sketch_y+bs_sketch_height+labelyoffset+65,150);
  reflectivity_m2_slider.setValueType(GWSlider.DECIMAL);
  reflectivity_m2_slider.setLimits(0.5f,0f,1f);
  reflectivity_m2_slider.setRenderValueLabel(false); 
  reflectivity_m2_slider.setValue(pow(beamsplitter_sketch.getM2Reflectivity(),2));

  rbs_label = new GLabel(this,"beam-splitter", bs_sketch_x+RSoffset1-90,bs_sketch_y+bs_sketch_height+labelyoffset+20,300);
  rm1_label = new GLabel(this,"first mirror",  bs_sketch_x+RSoffset2+160,bs_sketch_y+bs_sketch_height+labelyoffset+20,300);
  rm2_label = new GLabel(this,"second mirror", bs_sketch_x+RSoffset2+160,bs_sketch_y+bs_sketch_height+labelyoffset+60,300);

  beamsplitter_sketch.em_play();
}

void draw() {
  //  setBackgroundImage();
  background(200);
  beamsplitter_sketch.redraw();
  gw_logo.draw();
}


void handleSliderEvents(GSlider slider) {
  if (mousePressed == true)
    beamsplitter_sketch.em_pause();
  if (slider == cavity2_length_slider) {
    beamsplitter_sketch.setCavity2Length(slider.getValuef());
    float precision = 1000;
    float cav_dif = (slider.getValuef()-beamsplitter_sketch.getCavity1Length())/beamsplitter_sketch.getWavelength();
    dif_cav_dist_text.setText(String.valueOf(round(precision*cav_dif)/precision));
  }
  else if (slider == reflectivity_bs_slider)
    beamsplitter_sketch.setBSReflectivity(slider.getValuef());
  else if (slider == reflectivity_m1_slider)
    beamsplitter_sketch.setM1Reflectivity(slider.getValuef());
  else if (slider == reflectivity_m2_slider)
    beamsplitter_sketch.setM2Reflectivity(slider.getValuef());
}


void mouseReleased() {
  if (!paused)
    beamsplitter_sketch.em_play();
}

void handleTextFieldEvents(GTextField tfield) {
  String line1="";
  float temp_val;
  switch(dif_cav_dist_text.eventType) {
  case GTextField.CHANGED:
    break;
  case GTextField.SET:
    break;
  case GTextField.ENTERED:
    line1 = dif_cav_dist_text.getText();
    temp_val = parseFloat(line1)*beamsplitter_sketch.getWavelength()+beamsplitter_sketch.getCavity1Length();
    cavity2_length_slider.setValue(temp_val);
    beamsplitter_sketch.setCavity2Length(temp_val);
    break;
  }
}

public void keyPressed() {		
  if(key == ' ') {
    if (paused==true) {
      unpause();
    }   
    else {
      pause();
    }
  }
}

void handleCheckboxEvents(GCheckbox checkbox) {
  paused=chkContinous.isSelected();
  if(paused) {
    pause();
  }
  else {
    unpause();
  }
}

void pause() {
  paused=true;
  beamsplitter_sketch.em_pause();
  chkContinous.setSelected(true);
}

void unpause() {
  paused=false;
  beamsplitter_sketch.em_play();      
  chkContinous.setSelected(false);
}

/*

 static public void main(String args[]) {
 // use this if fullscreen library is NOT used:
 PApplet.main(new String[] { 
 "--present", "--bgcolor=#666666", "--hide-stop","Michelson01"});
 // use the following line if the fullscreen library is used:
 //PApplet.main(new String[] {
 //  "Michelson01"    });
 }
 */

